/****************************************************************************
 * This file is part of the program                                         *
 * mcmcsalut (Markov Chain Monte Carlo Sequence ALignment UTility) for      *
 * joint sampling alignments and mutation parameter values for two given    *
 * DNA-sequences.                                                           *
 *                                                                          *
 *    Copyright (C) 2000  Dirk Metzler                                      *
 *    e-mail: dmetzler@math.uni-frankfurt.de                                * 
 *                                                                          *
 *  This program is free software; you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation; either version 2 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with this program; if not, write to the                           *
 *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,             *
 *  Boston, MA  02111-1307  USA                                             *
 ****************************************************************************/

enum zustand {PAARUNG=0,A_GAP=1,B_GAP=2,START=3,ENDE=4,NOSTATE=5};

// class indelproc {
// protected:
//     double log_ueb_wkeiten[4][5];
//     virtual void update_log_ueb_wkeiten()=0;
// public:
//     virtual double log_wkeit(zustand a, zustand b) const
// 	/* Gibt die log_wkeit fuer einen Uebergang a->b zurueck.
// 	 */=0;
// };

// class TKFindelproc {
// private:
//     void update_log_ueb_wkeiten();
// public:
//     double log_wkeit(zustand a, zustand b) const
// 	/* Gibt die log_wkeit fuer einen Uebergang a->b zurueck.
// 	 */;
// };

class fragment_indelproc /*: public indelproc */{
private:
    double log_ueb_wkeiten[4][5];
    double lambda, // indel-rate
	gamma; // erwartete Laenge eines Fragments
    void update_log_ueb_wkeiten();
public:
    fragment_indelproc(double lam, double gam);
    fragment_indelproc(const fragment_indelproc& fip);
    fragment_indelproc& operator=(const fragment_indelproc& fip);
    double get_lambda() const { return lambda; }
    double get_gamma() const { return gamma; }
    void set(double lam, double gam) { lambda=lam; gamma=gam; update_log_ueb_wkeiten(); }
    double log_wkeit(zustand a, zustand b) const { 
	/* Gibt die log_wkeit fuer einen Uebergang a->b zurueck.*/ 
	if (a>3 || b>4) { 
	    cerr << "unerlaubter Zustand in fragment_indelproc::log_wkeit\n"; exit(1); } 
	return log_ueb_wkeiten[a][b]; }
};

ostream& operator<<(ostream& ostr, const fragment_indelproc& fip);
