#!/usr/bin/env wish
#
#  This file is part of the program mcmcsalut for joint sampling alignments  
#  and mutation parameter values for two given DNA-sequences.               
#                                                                         
#     Copyright (C) 2000  Dirk Metzler                                      
#     e-mail: dmetzler@math.uni-frankfurt.de                                 
#                                                                         
#  This program is free software; you can redistribute it and/or modify    
#  it under the terms of the GNU General Public License as published by    
#  the Free Software Foundation; either version 2 of the License, or       
#  (at your option) any later version.                                     
#                                                                        
#  This program is distributed in the hope that it will be useful,         
#  but WITHOUT ANY WARRANTY; without even the implied warranty of          
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           
#  GNU General Public License for more details.                            
#                                                                          
#  You should have received a copy of the GNU General Public License       
#  along with this program; if not, write to the                           
#  Free Software Foundation, Inc., 59 Temple Place, Suite 330,             
#  Boston, MA  02111-1307  USA                                             
#

wm minsize . 500 300

proc speichere {} {
    global seq1dat seq1dat
    global seq2dat seq2dat
    global ctrldat ctrldat
    global paramdat paramdat
    global start_algn start_algn
    global sub_rate sub_rate
    global ins_rate ins_rate
    global frag_len frag_len
    global anz_burn anz_burn
    global anz_runs anz_runs
    global anz_steps anz_steps
    global resamp resamp 
    global pA pA
    global pC pC
    global pG pC
    global pT pT
    global AG_rate AG_rate
    global CT_rate CT_rate
    global pr_frag_len pr_frag_len
    global fix_sir fix_sir
    global fix_GC fix_GC
    global fix_AzuT fix_AzuT
    global fix_CzuG  fix_CzuG
    global fix_subrate fix_subrate
    global fix_TizuTv  fix_TizuTv
    global fix_PuzuPy fix_PuzuPy
    global fix_lambda fix_lambda
    global fix_gamma   fix_gamma
    
    set ausgabe [open $ctrldat w]
    puts $ausgabe $seq1dat
    puts $ausgabe $seq2dat
    puts $ausgabe $paramdat
    puts $ausgabe [ switch $start_algn {
	given { format 0 }
	sampled { format 1 }
	ml { format 2 }
	givenfix { format 3 }
	default { puts "error in ausgabe!"} 
      }
    ]
    puts $ausgabe $pA
    puts $ausgabe $pC
    puts $ausgabe $pG
    puts $ausgabe $pT
    puts $ausgabe $sub_rate
    puts $ausgabe $AG_rate
    puts $ausgabe $CT_rate
    puts $ausgabe $ins_rate
    puts $ausgabe $frag_len
    puts $ausgabe $fix_sir
    if {$fix_sir==0} {
	puts $ausgabe $fix_GC
	puts $ausgabe $fix_AzuT
	puts $ausgabe $fix_CzuG
	puts $ausgabe $fix_subrate
	puts $ausgabe $fix_TizuTv
	puts $ausgabe $fix_PuzuPy
	puts $ausgabe $fix_lambda
	puts $ausgabe $fix_gamma
    }
    if {$fix_gamma==0} {
	puts $ausgabe $pr_frag_len
    }
    puts $ausgabe $anz_burn
    puts $ausgabe $anz_runs
    puts $ausgabe $anz_steps
    puts $ausgabe $resamp 
    close $ausgabe
}

proc liesdat {datei} {
    global seq1dat seq1dat
    global seq2dat seq2dat
    global ctrldat ctrldat
    global paramdat paramdat
    global start_algn start_algn
    global sub_rate sub_rate
    global ins_rate ins_rate
    global frag_len frag_len
    global anz_burn anz_burn
    global anz_runs anz_runs
    global anz_steps anz_steps
    global resamp resamp 
    global pA pA
    global pC pC
    global pG pC
    global pT pT
    global AG_rate AG_rate
    global CT_rate CT_rate
    global pr_frag_len pr_frag_len
    global fix_sir fix_sir
    global fix_GC fix_GC
    global fix_AzuT fix_AzuT
    global fix_CzuG  fix_CzuG
    global fix_subrate fix_subrate
    global fix_TizuTv  fix_TizuTv
    global fix_PuzuPy fix_PuzuPy
    global fix_lambda fix_lambda
    global fix_gamma   fix_gamma
    set ctrldat $datei
    if {[file exists $datei]==1} {
	set indat [open $datei r]    
	gets $indat seq1dat
	gets $indat seq2dat
	gets $indat paramdat
	gets $indat start_algn_num
	set start_algn [ switch $start_algn_num {
	    0 {format given}
	    1 {format sampled} 
	    2 {format  ml }
	    3 {format givenfix }
	    default { gets "error in ausgabe!"} 
	} ]
	gets $indat pA
	gets $indat pC
	gets $indat pG
	gets $indat pT
	gets $indat sub_rate
	gets $indat AG_rate
	gets $indat CT_rate
	gets $indat ins_rate
	gets $indat frag_len
	gets $indat fix_sir
	if {$fix_sir==0} {
	    gets $indat fix_GC
	    gets $indat fix_AzuT
	    gets $indat fix_CzuG
	    gets $indat fix_subrate
	    gets $indat fix_TizuTv
	    gets $indat fix_PuzuPy
	    gets $indat fix_lambda
	    gets $indat fix_gamma
	} {
	    set fix_GC 1
	    set fix_AzuT 1
	    set fix_CzuG 1
	    set fix_subrate 0
	    set fix_TizuTv 1
	    set fix_PuzuPy 1
	    set fix_lambda 0
	    set fix_gamma 1
	}
	if {$fix_gamma==0} {
	    gets $indat pr_frag_len
	}
	gets $indat anz_burn
	gets $indat anz_runs
	gets $indat anz_steps
	gets $indat resamp 
	close $indat
    }
}

# Filenames

set seq1dat seq1
set seq2dat seq2
if {$argc==0} {set ctrldat salut.ctr} {set ctrldat [lindex $argv 0]}
set paramdat paramsamp

frame .fr_files -relief sunken -borderwidth 1 
pack .fr_files  -fill x -expand 1
frame .fr_files.1st
frame .fr_files.2nd
pack .fr_files.1st .fr_files.2nd -in .fr_files 
frame .fr_files.3rd
frame .fr_files.4th
pack .fr_files.3rd .fr_files.4th -in .fr_files
label .l_seq1 -text "file with sequence 1" -width 30
entry .e_seq1 -textvariable seq1dat -width 30 -bg white
button .b_seq1 -text browse -command \
    {set seq1dat [tk_getOpenFile]}
pack .b_seq1 .e_seq1  .l_seq1 -side right -in .fr_files.1st
label .l_seq2 -text "file with sequence 2" -width 30
entry .e_seq2 -textvariable seq2dat -width 30 -bg white
button .b_seq2 -text browse -command \
    {set seq2dat [tk_getOpenFile]}
pack  .b_seq2 .e_seq2 .l_seq2 -side right -in .fr_files.2nd
label .l_ctrl -text "file for control instructions" -width 30
entry .e_ctrl -textvariable ctrldat -width 30 -bg white
button .b_ctrl -text browse -command {
    set ctrldat [tk_getOpenFile -initialfile $ctrldat]
    liesdat $ctrldat
}
bind .e_ctrl <FocusOut> {
    liesdat $ctrldat
}
pack .b_ctrl .e_ctrl  .l_ctrl -in .fr_files.3rd -side right
label .l_out -text "output file for parameters samples" -width 30
entry .e_out -textvariable paramdat -width 30 -bg white
button .b_out -text browse -command \
    {set paramdat [tk_getSaveFile -initialfile $paramdat]}
pack .b_out .e_out .l_out -in .fr_files.4th -side right

# Start Alignment

frame .fr_start -relief sunken -borderwidth 1
pack .fr_start  -fill x
frame .fr_start.align 
pack .fr_start.align -in  .fr_start -side left -fill x -expand 1
label .l_start_align -text "Start with..." -anchor w
pack .l_start_align -in .fr_start.align -fill x
set start_algn given
radiobutton .rb_given -variable start_algn -value given \
    -text "... given alignment" -anchor w 
radiobutton .rb_givenfix -variable start_algn -value givenfix \
    -text "... given alignment; keep fixed" -anchor w 
radiobutton .rb_sampled -variable start_algn -value sampled \
    -text "... sampled alignment" -anchor w
radiobutton .rb_ml -variable start_algn -value ml \
    -text "... most probable alignment" -anchor w 
pack .rb_given .rb_sampled .rb_ml .rb_givenfix -in .fr_start.align  -fill x

# Start Parameters
frame .fr_start.param 
pack .fr_start.param -in  .fr_start -side left -expand 1 -fill x  -fill y 
label .l_start_param -text "Start parameters"
pack .l_start_param -in .fr_start.param -fill x 
frame .fr_start.param.0
frame .fr_start.param.1 
frame .fr_start.param.2
frame .fr_start.param.3
pack .fr_start.param.0 .fr_start.param.1 .fr_start.param.2 .fr_start.param.3 -in .fr_start.param -fill y -anchor e
set pA 0.25
set pC 0.25
set pG 0.25
set pT 0.25
label .l_bfA -text "base frequencies: A"
entry .e_pA -textvariable pA -bg white -width 6
label .l_bfC -text " C"
entry .e_pC -textvariable pC -bg white -width 6
label .l_bfG -text " G"
entry .e_pG -textvariable pG -bg white -width 6
label .l_bfT -text " T"
entry .e_pT -textvariable pT -bg white -width 6
pack .l_bfA .e_pA .l_bfC .e_pC .l_bfG .e_pG .l_bfT .e_pT -in .fr_start.param.0 -side left 
set sub_rate 0.01
label .l_sub -text "substitution rates: general"
entry .e_sub -textvariable sub_rate -bg white -width 6
pack .l_sub  -in .fr_start.param.1 -side left  -expand 1 -fill y 
pack .e_sub  -in .fr_start.param.1 -side left -anchor e
set AG_rate 0.02
label .l_subAG -text " A<->G"
entry .e_subAG -textvariable AG_rate -bg white -width 6
pack .l_subAG  -in .fr_start.param.1 -side left  -expand 1 -fill y 
pack .e_subAG  -in .fr_start.param.1 -side left -anchor e
set CT_rate 0.02
label .l_subCT -text " C<->T"
entry .e_subCT -textvariable CT_rate -bg white -width 6
pack .l_subCT  -in .fr_start.param.1 -side left  -expand 1 -fill y 
pack .e_subCT  -in .fr_start.param.1 -side left -anchor e
label .l_indel -text "  indel rate (lambda)"
set ins_rate 0.001
entry .e_indel -textvariable ins_rate -bg white -width 6
pack  .l_indel -in .fr_start.param.2 -side left  -expand 1 -fill y 
pack  .e_indel -in .fr_start.param.2 -side left -anchor e
label .l_frag -text " expected fragment length (gamma)"
set frag_len 1.0
entry .e_frag -textvariable frag_len -bg white -width 6
pack  .l_frag -in .fr_start.param.2 -side left  -expand 1 -fill y 
pack  .e_frag -in .fr_start.param.2 -side left -anchor e
label .l_fragpr -text "expectation of prior on (gamma-1)"
set pr_frag_len 1.0
entry .e_fragpr -textvariable pr_frag_len -bg white -width 6
pack  .l_fragpr -in .fr_start.param.3 -side left  -expand 1 -fill y 
pack  .e_fragpr -in .fr_start.param.3 -side left -anchor e

# to keep fixed

frame .fr_fix -relief sunken  -borderwidth 1 
pack .fr_fix -expand 1 -fill x
frame .fr_fix.l -relief sunken 
frame .fr_fix.r -relief sunken  
pack .fr_fix.l .fr_fix.r -in .fr_fix -expand 1
set fix_sir 0
set fix_GC 0
set fix_AzuT 0
set fix_CzuG 0
set fix_subrate 0
set fix_TizuTv 0
set fix_PuzuPy 0
set fix_lambda 0
set fix_gamma 0
checkbutton .cb_sir -variable fix_sir -text "subst<->indel ratio" -command {
    if { $fix_sir==1 } {
	set fix_GC 1
	set fix_AzuT 1
	set fix_CzuG 1
	set fix_subrate 0
	set fix_TizuTv 1
	set fix_PuzuPy 1
	set fix_lambda 0
	set fix_gamma 1
    }
}
checkbutton .cb_GC -variable fix_GC  -text "GC:AT" -command {
    if { $fix_GC==0 } {
	set fix_sir 0
    }
}
checkbutton .cb_AzuT -variable fix_AzuT  -text "A:T" -command {
    if { $fix_AzuT==0 } {
	set fix_sir 0
    }
}
checkbutton .cb_CzuG -variable fix_CzuG  -text "C:G" -command {
    if { $fix_CzuG==0 } {
	set fix_sir 0
    }
}
checkbutton .cb_subrate -variable fix_subrate  -text "subrate" -command {
    if { $fix_subrate==1 } {
	set fix_sir 0
    }
}
checkbutton .cb_TizuTv -variable fix_TizuTv  -text "Ti:Tv" -command {
    if { $fix_TizuTv==0 } {
	set fix_sir 0
    }
}
checkbutton .cb_PuzuPy -variable fix_PuzuPy  -text "Pu:Py" -command {
    if { $fix_PuzuPy==0 } {
	set fix_sir 0
    }
}
checkbutton .cb_lambda -variable fix_lambda  -text "lambda" -command {
    if { $fix_lambda==1 } {
	set fix_sir 0
    }
}
checkbutton .cb_gamma -variable fix_gamma  -text "gamma" -command {
    if { $fix_gamma==0 } {
	set fix_sir 0
    }
}
label .l_fix -text "keep fixed:"
pack .l_fix .cb_sir .cb_GC .cb_AzuT .cb_CzuG -in .fr_fix.l -side left
pack .cb_subrate .cb_TizuTv .cb_PuzuPy  .cb_lambda .cb_gamma -in .fr_fix.r -side left

# Numbers of Runs
frame .fr_runs
pack .fr_runs -expand 1 -fill x
frame .fr_runs.anz -borderwidth 1 -relief sunken 
pack .fr_runs.anz -in .fr_runs -side left -fill x 
frame .fr_runs.1
frame .fr_runs.2
frame .fr_runs.3
frame .fr_runs.4
pack .fr_runs.1 .fr_runs.2 .fr_runs.3 .fr_runs.4 -in  .fr_runs.anz -fill x 
label .l_burn -text "number of burn-in runs : "
set anz_burn 10000
entry .e_burn -textvariable anz_burn -bg white
pack .e_burn .l_burn -in .fr_runs.1 -side right -fill x  -anchor e
label .l_runs -text "number of runs after burn-in : "
set anz_runs 100000
entry .e_runs -textvariable anz_runs -bg white
pack .e_runs .l_runs -in .fr_runs.2 -side right -fill x -anchor e
label .l_steps -text "number of runs for each output : "
set anz_steps 1000
entry .e_steps -textvariable anz_steps -bg white
pack .e_steps .l_steps -in .fr_runs.3 -side right -fill x -anchor e
label .l_resamp -text "range of resampling : "
set resamp 30
entry .e_resamp -textvariable resamp -bg white
pack .e_resamp .l_resamp -in .fr_runs.4 -side right -fill x -anchor e

# Save and Exit Buttons
frame .fr_runs.buttons -borderwidth 1 -relief sunken
pack .fr_runs.buttons -in .fr_runs -side left -expand 1  -fill y -fill x
button .savebutton -command speichere -text "save"
pack .savebutton -in .fr_runs.buttons -expand 1  -fill y -fill x
button .exitbutton -command {speichere; exit}  -text "save and exit"
pack .exitbutton -in .fr_runs.buttons -expand 1 -fill y -fill x
button .se_button -command exit -text "exit without saving"
pack .se_button -in .fr_runs.buttons -expand 1  -fill y -fill x

liesdat $ctrldat
