/****************************************************************************
 * This file is part of the program                                         *
 * mcmcsalut (Markov Chain Monte Carlo Sequence ALignment UTility)          *
 * for joint sampling alignments and mutation parameter values for two      *
 * given DNA-sequences.                                                     *
 *                                                                          *
 *    Copyright (C) 2000  Dirk Metzler                                      *
 *    e-mail: dmetzler@math.uni-frankfurt.de                                * 
 *                                                                          *
 *  This program is free software; you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation; either version 2 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with this program; if not, write to the                           *
 *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,             *
 *  Boston, MA  02111-1307  USA                                             *
 ****************************************************************************/

double rand_exp(double x) 
    /* gibt exp(x)-verteilte ZVE zurueck.
     * (Erwartungswert 1/x)
     */; 

// class proposal_chain {
//     /* The proposal chain of the Metropolis-Hastings-Algorithm
//      */
// public :
//     virtual double quot_prior() const
// 	/* liefert den Quotienten zwischen der Dichte der a-priori-Verteilung
// 	 * bezgl. der vorlaeufigen und der aktuellen Parameter
// 	 */=0;
//     virtual double uebergangs_quotient() const
// 	/* liefert p(vorlaeufig->aktuell)/p(aktuell->vorlaeufig)
// 	 * (wichtig fuer Metropolis-Hastings)
// 	 */=0; 
//     virtual void vorschlag() 
// 	/* setzt vorlaeufige Parameter gemaess der proposal chain
// 	 * aus den alten Parametern
// 	 */=0;
//     virtual void akzeptiere()
// 	/* waehlt die vorlaeufigen Parameter als aktuelle
// 	 */=0;
  
// };

class substproc_proposal_chain /*: public proposal_chain */{
    substproc aktuell, vorlaeufig;
    bool fix_GC_cont, fix_A_zu_T, fix_C_zu_G, fix_allgrate, 
	fix_Ti_zu_Tv, fix_Pu_zu_Py;
public :
    substproc_proposal_chain(const substproc & sp, const vector<bool> & fix);
    double quot_prior() const
	/* liefert den Quotienten zwischen der Dichte der a-priori-Verteilung
	 * bezgl. der vorlaeufigen und der aktuellen Parameter
	 */;
    double uebergangs_quotient() const
	/* liefert p(vorlaeufig->aktuell)/p(aktuell->vorlaeufig)
	 * (wichtig fuer Metropolis-Hastings)
	 */; 
    void vorschlag() 
	/* findet ein einen neuen Kandidaten fuer vorlaeufig
	 */;   
    void vorschlag(double t) 
	/* als Kandidaten fuer vorlaeufig werden die aktuellen
	 * Substitutionsraten um den Faktor t
	 * gestreckt.
	 */;
    void akzeptiere()
	/* waehlt die vorlaeufigen Parameter als aktuelle
	 */;
    substproc get_aktuell() const {return aktuell;}
    substproc get_vorlauf() const {return vorlaeufig;}
};

class fragindel_proposal_chain /*: public proposal_chain */ {
    fragment_indelproc aktuell, vorlaeufig;
    double prior_gamma_erw;
    bool fix_lambda, fix_gamma;
public :
    fragindel_proposal_chain(const fragment_indelproc & fip,
			     const vector<bool> & fix, 
			     double priorgamma);
    double quot_prior() const
	/* liefert den Quotienten zwischen der Dichte der a-priori-Verteilung
	 * bezgl. der vorlaeufigen und der aktuellen Parameter
	 */;
    double uebergangs_quotient() const
	/* liefert p(vorlaeufig->aktuell)/p(aktuell->vorlaeufig)
	 * (wichtig fuer Metropolis-Hastings)
	 */;
    void vorschlag() 
	/* findet einen Kandidaten fuer vorlaeufig
	 */;   
    void vorschlag(double t) 
	/* als Kandidaten fuer vorlaeufig wird lambda um den Faktor
	 * gestreckt.
	 */;
    void akzeptiere()
	/* waehlt die vorlaeufigen Parameter als aktuelle
	 */;
    fragment_indelproc get_aktuell() const {return aktuell;}
    fragment_indelproc get_vorlauf() const {return vorlaeufig;}
};
